/**
 * Orthanc - A Lightweight, RESTful DICOM Store
 * Copyright (C) 2012-2016 Sebastien Jodogne, Medical Physics
 * Department, University Hospital of Liege, Belgium
 * Copyright (C) 2017-2020 Osimis S.A., Belgium
 *
 * This program is free software: you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * In addition, as a special exception, the copyright holders of this
 * program give permission to link the code of its release with the
 * OpenSSL project's "OpenSSL" library (or with modified versions of it
 * that use the same license as the "OpenSSL" library), and distribute
 * the linked executables. You must obey the GNU General Public License
 * in all respects for all of the code used other than "OpenSSL". If you
 * modify file(s) with this exception, you may extend this exception to
 * your version of the file(s), but you are not obligated to do so. If
 * you do not wish to do so, delete this exception statement from your
 * version. If you delete this exception statement from all source files
 * in the program, then also delete it here.
 * 
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 **/


#pragma once

#if !defined(ORTHANC_SANDBOXED)
#  error The macro ORTHANC_SANDBOXED must be defined
#endif

#if ORTHANC_SANDBOXED == 1
#  error The class SharedArchive cannot be used in sandboxed environments
#endif

#include "LeastRecentlyUsedIndex.h"
#include "../IDynamicObject.h"

#include <map>
#include <boost/thread.hpp>

namespace Orthanc
{
  class SharedArchive : public boost::noncopyable
  {
  private:
    typedef std::map<std::string, IDynamicObject*>  Archive;

    size_t         maxSize_;
    boost::mutex   mutex_;
    Archive        archive_;
    LeastRecentlyUsedIndex<std::string> lru_;

    void RemoveInternal(const std::string& id);

  public:
    class Accessor : public boost::noncopyable
    {
    private:
      boost::mutex::scoped_lock  lock_;
      IDynamicObject*            item_;

    public:
      Accessor(SharedArchive& that,
               const std::string& id);

      bool IsValid() const
      {
        return item_ != NULL;
      }
      
      IDynamicObject& GetItem() const;
    };


    SharedArchive(size_t maxSize);

    ~SharedArchive();

    std::string Add(IDynamicObject* obj);  // Takes the ownership

    void Remove(const std::string& id);

    void List(std::list<std::string>& items);
  };
}


